@@ -4,18 +4,20 @@
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
  */
-#include "loader_p.h"
-#include "settings_p.h"
 #include "client_p.h"
+#include "loader_p.h"
+#include "settingsimpl_p.h"
 #include "spellerplugin_p.h"
 
-#include <QHash>
-#include <QMap>
-#include <QLocale>
-#include <QCoreApplication>
-#include <QPluginLoader>
 #include "core_debug.h"
+
+#include <QCoreApplication>
 #include <QDir>
+#include <QHash>
+#include <QLocale>
+#include <QMap>
+#include <QPluginLoader>
+#include <QVector>
 
 #include <algorithm>
 
@@ -26,18 +28,21 @@
 #endif
 #endif
 
-namespace Sonnet {
+namespace Sonnet
+{
 class LoaderPrivate
 {
 public:
-    Settings *settings;
+    SettingsImpl *settings;
 
     // <language, Clients with that language >
-    QMap<QString, QVector<Client *> > languageClients;
+    QMap<QString, QVector<Client *>> languageClients;
     QStringList clients;
 
+    QSet<QString> loadedPlugins;
+
     QStringList languagesNameCache;
-    QHash<QString, QSharedPointer<SpellerPlugin> > spellerCache;
+    QHash<QString, QSharedPointer<SpellerPlugin>> spellerCache;
 };
 
 Q_GLOBAL_STATIC(Loader, s_loader)
@@ -54,7 +59,7 @@ Loader *Loader::openLoader()
 Loader::Loader()
     : d(new LoaderPrivate)
 {
-    d->settings = new Settings(this);
+    d->settings = new SettingsImpl(this);
     d->settings->restore();
     loadPlugins();
 }
@@ -78,8 +83,12 @@ SpellerPlugin *Loader::createSpeller(const QString &language, const QString &cli
 
     auto clientsItr = d->languageClients.constFind(plang);
     if (clientsItr == d->languageClients.constEnd()) {
+        if (language.isEmpty()) {
+            qCWarning(SONNET_LOG_CORE) << "No language dictionaries for the language:" << plang << "trying to load en_US as default";
+            return createSpeller(QStringLiteral("en_US"), clientName);
+        }
         qCWarning(SONNET_LOG_CORE) << "No language dictionaries for the language:" << plang;
-        emit loadingDictionaryFailed(plang);
+        Q_EMIT loadingDictionaryFailed(plang);
         return nullptr;
     }
 
@@ -90,8 +99,9 @@ SpellerPlugin *Loader::createSpeller(const QString &language, const QString &cli
         if (!backend.isEmpty()) {
             // check if the default client supports the requested language;
             // if it does it will be an element of lClients.
-            bool unknown = !std::any_of(lClients.constBegin(), lClients.constEnd(), [backend] (const Client *client) {
-                    return client->name() == backend; });
+            bool unknown = !std::any_of(lClients.constBegin(), lClients.constEnd(), [backend](const Client *client) {
+                return client->name() == backend;
+            });
             if (unknown) {
                 qCWarning(SONNET_LOG_CORE) << "Default client" << backend << "doesn't support language:" << plang;
                 backend = QString();
@@ -109,8 +119,8 @@ SpellerPlugin *Loader::createSpeller(const QString &language, const QString &cli
                 return dict;
             }
         } else {
-            //the first one is the one with the highest
-            //reliability
+            // the first one is the one with the highest
+            // reliability
             SpellerPlugin *dict = item->createSpeller(plang);
             qCDebug(SONNET_LOG_CORE) << "Using the" << item->name() << "plugin for language" << plang;
             return dict;
@@ -147,16 +157,16 @@ QStringList Loader::languages() const
 
 QString Loader::languageNameForCode(const QString &langCode) const
 {
-    QString currentDictionary = langCode,   // e.g. en_GB-ize-wo_accents
-            isoCode,            // locale ISO name
-            variantName,         // dictionary variant name e.g. w_accents
-            localizedLang,       // localized language
-            localizedCountry,    // localized country
-            localizedVariant;
+    QString currentDictionary = langCode; // e.g. en_GB-ize-wo_accents
+    QString isoCode; // locale ISO name
+    QString variantName; // dictionary variant name e.g. w_accents
+    QString localizedLang; // localized language
+    QString localizedCountry; // localized country
+    QString localizedVariant;
     QByteArray variantEnglish; // dictionary variant in English
 
-    int minusPos,          // position of "-" char
-        variantCount = 0;  // used to iterate over variantList
+    int minusPos; // position of "-" char
+    int variantCount = 0; // used to iterate over variantList
 
     struct variantListType {
         const char *variantShortName;
@@ -172,41 +182,28 @@ QString Loader::languageNameForCode(const QString &langCode) const
 #undef QT_TRANSLATE_NOOP3
 #define QT_TRANSLATE_NOOP3(a, b, c) b
 
-    const variantListType variantList[] = {
-        { "40", QT_TRANSLATE_NOOP3("Sonnet::Loader", "40", "dictionary variant") },             // what does 40 mean?
-        { "60", QT_TRANSLATE_NOOP3("Sonnet::Loader", "60", "dictionary variant") },             // what does 60 mean?
-        { "80", QT_TRANSLATE_NOOP3("Sonnet::Loader", "80", "dictionary variant") },             // what does 80 mean?
-        { "ise", QT_TRANSLATE_NOOP3("Sonnet::Loader", "-ise suffixes", "dictionary variant") },
-        { "ize", QT_TRANSLATE_NOOP3("Sonnet::Loader", "-ize suffixes", "dictionary variant") },
-        { "ise-w_accents", QT_TRANSLATE_NOOP3("Sonnet::Loader", "-ise suffixes and with accents",
-                                              "dictionary variant") },
-        { "ise-wo_accents", QT_TRANSLATE_NOOP3("Sonnet::Loader",
-                                               "-ise suffixes and without accents",
-                                               "dictionary variant") },
-        { "ize-w_accents", QT_TRANSLATE_NOOP3("Sonnet::Loader", "-ize suffixes and with accents",
-                                              "dictionary variant") },
-        { "ize-wo_accents", QT_TRANSLATE_NOOP3("Sonnet::Loader",
-                                               "-ize suffixes and without accents",
-                                               "dictionary variant") },
-        { "lrg", QT_TRANSLATE_NOOP3("Sonnet::Loader", "large", "dictionary variant") },
-        { "med", QT_TRANSLATE_NOOP3("Sonnet::Loader", "medium", "dictionary variant") },
-        { "sml", QT_TRANSLATE_NOOP3("Sonnet::Loader", "small", "dictionary variant") },
-        { "variant_0", QT_TRANSLATE_NOOP3("Sonnet::Loader", "variant 0", "dictionary variant") },
-        { "variant_1", QT_TRANSLATE_NOOP3("Sonnet::Loader", "variant 1", "dictionary variant") },
-        { "variant_2", QT_TRANSLATE_NOOP3("Sonnet::Loader", "variant 2", "dictionary variant") },
-        { "wo_accents",
-          QT_TRANSLATE_NOOP3("Sonnet::Loader", "without accents", "dictionary variant") },
-        { "w_accents", QT_TRANSLATE_NOOP3("Sonnet::Loader", "with accents", "dictionary variant") },
-        { "ye",
-          QT_TRANSLATE_NOOP3("Sonnet::Loader", "with ye, modern russian",
-                             "dictionary variant") },
-        { "yeyo", QT_TRANSLATE_NOOP3("Sonnet::Loader", "with yeyo, modern and old russian",
-                                     "dictionary variant") },
-        { "yo",
-          QT_TRANSLATE_NOOP3("Sonnet::Loader", "with yo, old russian", "dictionary variant") },
-        { "extended", QT_TRANSLATE_NOOP3("Sonnet::Loader", "extended", "dictionary variant") },
-        { nullptr, nullptr }
-    };
+    const variantListType variantList[] = {{"40", QT_TRANSLATE_NOOP3("Sonnet::Loader", "40", "dictionary variant")}, // what does 40 mean?
+                                           {"60", QT_TRANSLATE_NOOP3("Sonnet::Loader", "60", "dictionary variant")}, // what does 60 mean?
+                                           {"80", QT_TRANSLATE_NOOP3("Sonnet::Loader", "80", "dictionary variant")}, // what does 80 mean?
+                                           {"ise", QT_TRANSLATE_NOOP3("Sonnet::Loader", "-ise suffixes", "dictionary variant")},
+                                           {"ize", QT_TRANSLATE_NOOP3("Sonnet::Loader", "-ize suffixes", "dictionary variant")},
+                                           {"ise-w_accents", QT_TRANSLATE_NOOP3("Sonnet::Loader", "-ise suffixes and with accents", "dictionary variant")},
+                                           {"ise-wo_accents", QT_TRANSLATE_NOOP3("Sonnet::Loader", "-ise suffixes and without accents", "dictionary variant")},
+                                           {"ize-w_accents", QT_TRANSLATE_NOOP3("Sonnet::Loader", "-ize suffixes and with accents", "dictionary variant")},
+                                           {"ize-wo_accents", QT_TRANSLATE_NOOP3("Sonnet::Loader", "-ize suffixes and without accents", "dictionary variant")},
+                                           {"lrg", QT_TRANSLATE_NOOP3("Sonnet::Loader", "large", "dictionary variant")},
+                                           {"med", QT_TRANSLATE_NOOP3("Sonnet::Loader", "medium", "dictionary variant")},
+                                           {"sml", QT_TRANSLATE_NOOP3("Sonnet::Loader", "small", "dictionary variant")},
+                                           {"variant_0", QT_TRANSLATE_NOOP3("Sonnet::Loader", "variant 0", "dictionary variant")},
+                                           {"variant_1", QT_TRANSLATE_NOOP3("Sonnet::Loader", "variant 1", "dictionary variant")},
+                                           {"variant_2", QT_TRANSLATE_NOOP3("Sonnet::Loader", "variant 2", "dictionary variant")},
+                                           {"wo_accents", QT_TRANSLATE_NOOP3("Sonnet::Loader", "without accents", "dictionary variant")},
+                                           {"w_accents", QT_TRANSLATE_NOOP3("Sonnet::Loader", "with accents", "dictionary variant")},
+                                           {"ye", QT_TRANSLATE_NOOP3("Sonnet::Loader", "with ye, modern russian", "dictionary variant")},
+                                           {"yeyo", QT_TRANSLATE_NOOP3("Sonnet::Loader", "with yeyo, modern and old russian", "dictionary variant")},
+                                           {"yo", QT_TRANSLATE_NOOP3("Sonnet::Loader", "with yo, old russian", "dictionary variant")},
+                                           {"extended", QT_TRANSLATE_NOOP3("Sonnet::Loader", "extended", "dictionary variant")},
+                                           {nullptr, nullptr}};
 
     minusPos = currentDictionary.indexOf(QLatin1Char('-'));
     if (minusPos != -1) {
@@ -235,15 +232,14 @@ QString Loader::languageNameForCode(const QString &langCode) const
     localizedLang = locale.nativeLanguageName();
 
     if (localizedLang.isEmpty() && localizedCountry.isEmpty()) {
-        return isoCode;    // We have nothing
+        return isoCode; // We have nothing
     }
 
     if (!localizedCountry.isEmpty() && !localizedVariant.isEmpty()) { // We have both a country name and a variant
-        return tr("%1 (%2) [%3]", "dictionary name; %1 = language name, %2 = country name and %3 = language variant name"
-                  ).arg(localizedLang, localizedCountry, localizedVariant);
+        return tr("%1 (%2) [%3]", "dictionary name; %1 = language name, %2 = country name and %3 = language variant name")
+            .arg(localizedLang, localizedCountry, localizedVariant);
     } else if (!localizedCountry.isEmpty()) { // We have a country name
-        return tr("%1 (%2)", "dictionary name; %1 = language name, %2 = country name"
-                  ).arg(localizedLang, localizedCountry);
+        return tr("%1 (%2)", "dictionary name; %1 = language name, %2 = country name").arg(localizedLang, localizedCountry);
     } else { // We only have a language name
         return localizedLang;
     }
@@ -267,7 +263,7 @@ QStringList Loader::languageNames() const
     return allLocalizedDictionaries;
 }
 
-Settings *Loader::settings() const
+SettingsImpl *Loader::settings() const
 {
     return d->settings;
 }
@@ -277,7 +273,6 @@ void Loader::loadPlugins()
 #ifndef SONNET_STATIC
     const QStringList libPaths = QCoreApplication::libraryPaths() << QStringLiteral(INSTALLATION_PLUGIN_PATH);
     const QLatin1String pathSuffix("/kf5/sonnet/");
-    int plugins = 0;
     for (const QString &libPath : libPaths) {
         QDir dir(libPath + pathSuffix);
         if (!dir.exists()) {
@@ -285,10 +280,10 @@ void Loader::loadPlugins()
         }
         for (const QString &fileName : dir.entryList(QDir::Files)) {
             loadPlugin(dir.absoluteFilePath(fileName));
-            plugins++;
         }
     }
-    if (plugins == 0) {
+
+    if (d->loadedPlugins.isEmpty()) {
         qCWarning(SONNET_LOG_CORE) << "Sonnet: No speller backends available!";
     }
 #else
@@ -304,10 +299,17 @@ void Loader::loadPlugin(const QString &pluginPath)
 #ifndef SONNET_STATIC
     QPluginLoader plugin(pluginPath);
     if (!plugin.load()) { // We do this separately for better error handling
-        qCWarning(SONNET_LOG_CORE) << "Sonnet: Unable to load plugin" << pluginPath << "Error:"
-                                   << plugin.errorString();
+        qCWarning(SONNET_LOG_CORE) << "Sonnet: Unable to load plugin" << pluginPath << "Error:" << plugin.errorString();
         return;
     }
+    const QString pluginIID = plugin.metaData()[QStringLiteral("IID")].toString();
+    if (!pluginIID.isEmpty()) {
+        if (d->loadedPlugins.contains(pluginIID)) {
+            qCDebug(SONNET_LOG_CORE) << "Skipping already loaded" << pluginPath;
+            return;
+        }
+        d->loadedPlugins.insert(pluginIID);
+    }
 
     Client *client = qobject_cast<Client *>(plugin.instance());
     if (!client) {
@@ -333,17 +335,17 @@ void Loader::loadPlugin(const QString &pluginPath)
     for (const QString &language : languages) {
         QVector<Client *> &languageClients = d->languageClients[language];
 
-        if (languageClients.isEmpty()
+        if (languageClients.isEmpty() //
             || client->reliability() < languageClients.first()->reliability()) {
-            languageClients.append(client);    // less reliable, to the end
+            languageClients.append(client); // less reliable, to the end
         } else {
-            languageClients.prepend(client);    // more reliable, to the front
+            languageClients.prepend(client); // more reliable, to the front
         }
     }
 }
 
 void Loader::changed()
 {
-    emit configurationChanged();
+    Q_EMIT configurationChanged();
 }
 }
